/* 
 * Copyright (c) 2002 Michal Moskal <malekith@pld-linux.org>
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Id: c_stdio.c,v 1.1 2002/06/04 11:54:12 malekith Exp $
 *
 * Standard library for Gont, Stdio module C implementation.
 */

#include <gont/runtime.h>
#include <gont/array.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

struct gont_file {
	FILE *fp;
	int p_kind;
};

extern gont_ptroff Std$$Sys_error;

static void sys_error(int err)
{
	gont_raise(gont_make_union(Std$$Sys_error, 
				   gont_cstring(strerror(err))));
}

static struct gont_file *gont_make_file(FILE *f)
{
	struct gont_file *gf;

	gf = gont_xmalloc0(sizeof(*gf));
	gf->fp = f;
	return gf;
}

struct gont_file *Stdio__fopen(struct gont_string *filename, 
			       struct gont_string *flags)
{
	FILE *f = fopen(filename->data, flags->data);
	
	if (f == NULL)
		sys_error(errno);
		
	return gont_make_file(f);
}

struct gont_file *Stdio__fdopen(gont_ptroff fd, struct gont_string *flags)
{
	FILE *f = fdopen(fd, flags->data);
	
	if (f == NULL)
		sys_error(errno);
		
	return gont_make_file(f);
}

struct gont_file *Stdio__popen(struct gont_string *cmd, 
			       struct gont_string *flags)
{
	FILE *f = popen(cmd->data, flags->data);
	struct gont_file *r;
	
	if (f == NULL)
		sys_error(errno);
		
	r = gont_make_file(f);
	r->p_kind = 1;

	return r;
}

gont_ptroff Stdio__pclose(struct gont_file *f)
{
	int r;
	
	if (f->fp == NULL || f->p_kind == 0)
		gont_invalid_argument("Stdio::pclose");
	r = pclose(f->fp);
	f->fp = NULL;
	return r;
}

void Stdio__fclose(struct gont_file *f)
{
	int err;

	if (f->fp == NULL)
		gont_invalid_argument("Stdio::fclose");
	
	err = fclose(f->fp);
	if (err)
		err = errno;
	
	f->fp = NULL;
	
	if (err)
		sys_error(err);
}

struct gont_string *Std__read_line_f(FILE *f);

struct gont_string *Stdio__fgets(struct gont_file *f)
{
	if (f->fp == NULL)
		gont_invalid_argument("Stdio::fgets");

	return Std__read_line_f(f->fp);
}

void Stdio__fputs(struct gont_file *f, struct gont_string *s)
{
	if (f->fp == NULL)
		gont_invalid_argument("Stdio::fputs");
	if (fputs(s->data, f->fp) == EOF)
		sys_error(errno);
}

gont_ptroff Stdio__feof(struct gont_file *f)
{
	if (f->fp == NULL)
		gont_invalid_argument("Stdio::feof");
	return feof(f->fp) ? 1 : 0;
}

gont_ptroff Stdio__ferror(struct gont_file *f)
{
	if (f->fp == NULL)
		gont_invalid_argument("Stdio::ferror");
	return ferror(f->fp) ? 1 : 0;
}

void Stdio__clearerr(struct gont_file *f)
{
	if (f->fp == NULL)
		gont_invalid_argument("Stdio::clearerr");
	clearerr(f->fp);
}

gont_ptroff Stdio__fileno(struct gont_file *f)
{
	if (f->fp == NULL)
		gont_invalid_argument("Stdio::fileno");
	return fileno(f->fp);
}

/* TODO:
struct gont_string *fread(struct gont_file *f, int cnt);
void fflush(struct gont_file *f);
void fseek(struct gont_file *f, int offset, fwhence whence);
int ftell(struct gont_file *f);
void rewind(struct gont_file *f);
void setlinebuf(struct gont_file *f);
void setnobuf(struct gont_file *f);
*/

/* File functions. */
void Stdio__remove(struct gont_string *filename)
{
	if (remove(filename->data))
		sys_error(errno);
}

void Stdio__rename(struct gont_string *oldname, struct gont_string *newname)
{
	if (rename(oldname->data, newname->data))
		sys_error(errno);
}
